@@ -1,12 +1,17 @@
 # -*- coding: utf-8 -*-
 
+from django.conf import settings
+if 'localeurl' in settings.INSTALLED_APPS:
+    from localeurl.models import patch_reverse
+    patch_reverse()
+    
 from django.test import TestCase
 from django.contrib.auth.models import User, Permission
 from django.core.urlresolvers import reverse
 from datetime import datetime
 from model_mommy import mommy
 from sanza.Crm import models
-from sanza.Emailing.models import Emailing, MagicLink, EmailingCounter
+from sanza.Emailing.models import Emailing, MagicLink
 from coop_cms.models import Newsletter
 from django.core import management
 from django.core import mail
@@ -14,6 +19,8 @@ from django.conf import settings
 from coop_cms import tests as coop_cms_tests
 from captcha.models import CaptchaStore
 from django.utils import timezone
+from django.contrib.sites.models import Site
+from bs4 import BeautifulSoup as BS4
 
 class BaseTestCase(TestCase):
 
@@ -31,7 +38,10 @@ class BaseTestCase(TestCase):
 class EmailingManagementTestCase(BaseTestCase):
     
     def now(self):
-        return datetime.now().replace(tzinfo=timezone.utc)
+        if settings.USE_TZ:
+            return datetime.now().replace(tzinfo=timezone.utc)
+        else:
+            return datetime.now()
     
     
     def test_view_newsletters_list(self):
@@ -157,18 +167,22 @@ class EmailingManagementTestCase(BaseTestCase):
         self.assertEqual(emailing.status, Emailing.STATUS_EDITING)
         self.assertEqual(emailing.scheduling_dt, None)
         
-        response = self.client.post(next_url)
+        response = self.client.post(next_url, data={"scheduling_dt": "01/01/2120 00:00"})
         self.assertEqual(200, response.status_code)
         emailing = Emailing.objects.get(id=emailing.id)
         self.assertEqual(emailing.status, Emailing.STATUS_EDITING)
         self.assertEqual(emailing.scheduling_dt, None)
         
-        response = self.client.post(next_url, data={"confirm": "1"})
+        response = self.client.post(next_url, data={"confirm": "1", "scheduling_dt": "01/01/2120 00:00"})
         self.assertEqual(200, response.status_code)
         emailing = Emailing.objects.get(id=emailing.id)
         self.assertEqual(emailing.status, Emailing.STATUS_SCHEDULED)
         self.assertNotEqual(emailing.scheduling_dt, None)
-        self.assertTrue(emailing.scheduling_dt > datetime.now().replace(tzinfo=timezone.get_current_timezone()))
+        if settings.USE_TZ:
+            self.assertTrue(emailing.scheduling_dt > datetime.now().replace(tzinfo=timezone.get_current_timezone()))
+        else:
+            self.assertTrue(emailing.scheduling_dt > datetime.now())
+            
         
         
     def test_cancel_sending(self):
@@ -210,8 +224,13 @@ class SendEmailingTest(BaseTestCase):
         settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'
         settings.COOP_CMS_REPLY_TO = 'titi@toto.fr'
         
+        site = Site.objects.get_current()
+        site.domain = settings.COOP_CMS_SITE_PREFIX
+        site.save()
+        
         
     def test_send_newsletter(self):
+            
         entity = mommy.make(models.Entity, name="my corp")
         
         names = ['alpha', 'beta', 'gamma']
@@ -225,8 +244,6 @@ class SendEmailingTest(BaseTestCase):
         }
         newsletter = mommy.make(Newsletter, **newsletter_data)
         
-        management.call_command('add_emailing_counter', 5, verbosity=0, interactive=False)
-        
         emailing = mommy.make(Emailing,
             newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
             scheduling_dt = datetime.now(), sending_dt = None)
@@ -244,11 +261,6 @@ class SendEmailingTest(BaseTestCase):
         self.assertEqual(emailing.send_to.count(), 0)
         self.assertEqual(emailing.sent_to.count(), len(contacts))
         
-        self.assertEqual(1, EmailingCounter.objects.count())
-        counter = EmailingCounter.objects.all()[0]
-        self.assertEqual(5, counter.total)
-        self.assertEqual(counter.total - len(contacts), counter.credit)
-        
         self.assertEqual(len(mail.outbox), len(contacts))
         
         outbox = list(mail.outbox)
@@ -284,266 +296,6 @@ class SendEmailingTest(BaseTestCase):
             c = models.Contact.objects.get(id=contact.id)
             self.assertEqual(c.action_set.count(), 1)
             self.assertEqual(c.action_set.all()[0].subject, email.subject)
-            
-            
-    def test_send_newsletter_limit(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        management.call_command('add_emailing_counter', 5, verbosity=0, interactive=False)
-        
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        
-        #check emailing status
-        self.assertEqual(emailing.status, Emailing.STATUS_SENDING)
-        self.assertEqual(emailing.sending_dt, None)
-        self.assertEqual(emailing.send_to.count(), 1)
-        self.assertEqual(emailing.sent_to.count(), 2)
-        
-        self.assertEqual(len(mail.outbox), emailing.sent_to.count())
-        
-        outbox = list(mail.outbox)
-        outbox.sort(key=lambda e: e.to)
-        contacts.sort(key=lambda c: c.get_email)
-        
-        for email, contact in zip(outbox, emailing.sent_to.all()):
-            self.assertEqual(email.to, [contact.get_email_address()])
-            
-        mail.outbox = []
-
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
-        self.assertNotEqual(emailing.sending_dt, None)
-        self.assertEqual(emailing.send_to.count(), 0)
-        self.assertEqual(emailing.sent_to.count(), 3)
-        self.assertEqual(len(mail.outbox), 1)
-        
-        self.assertEqual(1, EmailingCounter.objects.count())
-        counter = EmailingCounter.objects.all()[0]
-        self.assertEqual(5, counter.total)
-        self.assertEqual(counter.total - len(contacts), counter.credit)
-        
-    def test_send_newsletter_no_credit(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        
-        #check emailing status
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-    def test_send_newsletter_credit_missing(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma', 'delta', 'eta',
-            'one', 'two', 'three', 'four', 'five', 'un', 'deux']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        #for x in range(10):
-        management.call_command('add_emailing_counter', 10, verbosity=0, interactive=False)
-        
-        management.call_command('emailing_scheduler', 11, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        
-        #check emailing status
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-    def test_send_newsletter_credit_missing_new_credit(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        #Only 1 credit and try to send 3 emails -> credit missing
-        management.call_command('add_emailing_counter', 1, verbosity=0, interactive=False)
-        
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-        #Add 1 more credit and try to send 3 emails --> still 1 missing
-        management.call_command('add_emailing_counter', 1, verbosity=0, interactive=False)
-        
-        emailing.status = Emailing.STATUS_SCHEDULED
-        emailing.save()
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-        #Add 1 more -> 3 credits for emaisl to send -> Ok
-        management.call_command('add_emailing_counter', 1, verbosity=0, interactive=False)
-        
-        emailing.status = Emailing.STATUS_SCHEDULED
-        emailing.save()
-        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
-        self.assertEqual(len(mail.outbox), 3)
-        
-    def test_send_newsletter_credit_missing_new_emailing(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        emailing2 = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        emailing2.send_to.add(contacts[0])
-        emailing2.save()
-        
-        management.call_command('add_emailing_counter', 1, verbosity=0, interactive=False)
-        
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        
-        emailing2 = Emailing.objects.get(id=emailing2.id)
-        self.assertEqual(emailing2.status, Emailing.STATUS_SENT)
-        
-        self.assertEqual(len(mail.outbox), 1)
-        
-        emailing.status = Emailing.STATUS_SCHEDULED
-        emailing.save()
-        
-        management.call_command('add_emailing_counter', 3, verbosity=0, interactive=False)
-        management.call_command('emailing_scheduler', 10, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
-        
-        emailing2 = Emailing.objects.get(id=emailing2.id)
-        self.assertEqual(emailing2.status, Emailing.STATUS_SENT)
-        
-        self.assertEqual(len(mail.outbox), 4)
-        
-    
-    def test_send_newsletter_credit_missing_reset_before_resend(self):
-        entity = mommy.make(models.Entity, name="my corp")
-        names = ['alpha', 'beta', 'gamma']
-        contacts = [mommy.make(models.Contact, entity=entity,
-            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
-        newsletter_data = {
-            'subject': 'This is the subject',
-            'content': '<h2>Hello {fullname}!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
-            'template': 'test/newsletter_contact.html'
-        }
-        newsletter = mommy.make(Newsletter, **newsletter_data)
-        
-        emailing = mommy.make(Emailing,
-            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
-            scheduling_dt = datetime.now(), sending_dt = None)
-        for c in contacts:
-            emailing.send_to.add(c)
-        emailing.save()
-        
-        #Credit missing
-        management.call_command('add_emailing_counter', 1, verbosity=0, interactive=False)
-        management.call_command('emailing_scheduler', 2, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-        #Add credit
-        management.call_command('add_emailing_counter', 3, verbosity=0, interactive=False)
-        management.call_command('emailing_scheduler', 10, verbosity=0, interactive=False)
-        
-        emailing = Emailing.objects.get(id=emailing.id)
-        self.assertEqual(emailing.status, Emailing.STATUS_CREDIT_MISSING)
-        self.assertEqual(len(mail.outbox), 0)
-        
-        #Reset the credit_missing status
-        emailing.status = Emailing.STATUS_SCHEDULED
-        emailing.save()
-        management.call_command('emailing_scheduler', 10, verbosity=0, interactive=False)
-        
-        self.assertEqual(len(mail.outbox), 3)
-        
                 
     def test_view_magic_link(self):
         entity = mommy.make(models.Entity, name="my corp")
@@ -614,9 +366,6 @@ class NewsletterTest(coop_cms_tests.NewsletterTest):
 class SubscribeTest(TestCase):
     
     def setUp(self):
-        self.group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
-        self.group2 = mommy.make(models.Group, name="DEF", subscribe_form=True)
-        self.group3 = mommy.make(models.Group, name="GHI", subscribe_form=False)
         
         if not getattr(settings, 'SANZA_ALLOW_SINGLE_CONTACT', True):
             settings.SANZA_INDIVIDUAL_ENTITY_ID = models.EntityType.objects.create(name="particulier").id
@@ -628,17 +377,80 @@ class SubscribeTest(TestCase):
         response = self.client.get(url)
         self.assertEqual(200, response.status_code)
         
-        self.assertContains(response, self.group1.name)
-        self.assertContains(response, self.group2.name)
-        self.assertNotContains(response, self.group3.name)
+    def test_view_email_subscribe_newsletter(self):
+        url = reverse("emailing_email_subscribe_newsletter")
+        response = self.client.get(url)
+        self.assertEqual(200, response.status_code)
+        
+    def test_email_subscribe_newsletter(self):
+        url = reverse("emailing_email_subscribe_newsletter")
+        
+        data = {
+            'email': 'pdupond@apidev.fr',
+        }
+        
+        response = self.client.post(url, data=data, follow=False)
+        self.assertEqual(302, response.status_code)
+        self.assertEqual(models.Contact.objects.count(), 1)
+        
+        contact = models.Contact.objects.all()[0]
+        
+        self.assertNotEqual(contact.uuid, '')
+        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done'))>=0)
+        
+        self.assertEqual(contact.email, data['email'])
+        
+        self.assertEqual(len(mail.outbox), 2) #email verification
+        
+        verification_email = mail.outbox[1]
+        self.assertEqual(verification_email.to, [contact.email]) #email verification
+        url = reverse('emailing_email_verification', args=[contact.uuid])
+        email_content = verification_email.message().as_string().decode('utf-8')
+        self.assertTrue(email_content.find(url)>0) #email verification
+        
+        notification_email = mail.outbox[0]
+        self.assertEqual(notification_email.to, [settings.SANZA_NOTIFICATION_EMAIL])
+        
+    def test_email_subscribe_newsletter_no_email(self):
+        url = reverse("emailing_email_subscribe_newsletter")
+        
+        data = {
+            'email': '',
+        }
+        
+        response = self.client.post(url, data=data, follow=False)
+        self.assertEqual(200, response.status_code)
+        soup = BS4(response.content)
+        self.assertEqual(len(soup.select("ul.errorlist")), 1)
+        self.assertEqual(models.Contact.objects.count(), 0)
+        
+        self.assertEqual(len(mail.outbox), 0) #email verification
+        
+    def test_email_subscribe_newsletter_invalid_email(self):
+        url = reverse("emailing_email_subscribe_newsletter")
+        
+        data = {
+            'email': 'coucou',
+        }
+        
+        response = self.client.post(url, data=data, follow=False)
+        self.assertEqual(200, response.status_code)
+        soup = BS4(response.content)
+        self.assertEqual(len(soup.select("ul.errorlist")), 1)
+        self.assertEqual(models.Contact.objects.count(), 0)
+        
+        self.assertEqual(len(mail.outbox), 0) #email verification
+        
         
     def test_subscribe_newsletter_no_email(self):
+        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
+        
         url = reverse("emailing_subscribe_newsletter")
         
         data = {
             'lastname': 'Dupond',
             'firstname': 'Pierre',
-            'groups': str(self.group1.id),
+            'groups': str(group1.id),
         }
         self._patch_with_captcha(url, data)
         
@@ -649,15 +461,90 @@ class SubscribeTest(TestCase):
         
         self.assertEqual(len(mail.outbox), 0)
         
-        
-    def test_subscribe_newsletter_no_entity(self):
+    def test_subscribe_newsletter_message(self):
         url = reverse("emailing_subscribe_newsletter")
         
         data = {
             'entity_type': 0,
             'lastname': 'Dupond',
             'firstname': 'Pierre',
-            'groups': str(self.group1.id),
+            'email': "pierre.dupond@mon-mail.fr",
+            'message': "Hello",
+        }
+        self._patch_with_captcha(url, data)
+        
+        self.assertEqual(models.Contact.objects.count(), 0)
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(200, response.status_code)
+        self.assertEqual(models.Contact.objects.count(), 1)
+        contact = models.Contact.objects.all()[0]
+        
+        self.assertEqual(contact.email, data["email"])
+        self.assertEqual(contact.lastname, data["lastname"])
+        self.assertEqual(contact.firstname, data["firstname"])
+        
+        self.assertEqual(1, contact.action_set.count())
+        action = contact.action_set.all()[0]
+        self.assertEqual(data["message"], action.detail)
+        
+        self.assertEqual(len(mail.outbox), 2) #email verification
+        
+        verification_email = mail.outbox[1]
+        self.assertEqual(verification_email.to, [contact.email]) #email verification
+        url = reverse('emailing_email_verification', args=[contact.uuid])
+        email_content = verification_email.message().as_string().decode('utf-8')
+        self.assertTrue(email_content.find(url)>0) #email verification
+        
+        notification_email = mail.outbox[0]
+        self.assertEqual(notification_email.to, [settings.SANZA_NOTIFICATION_EMAIL])
+    
+    def test_subscribe_newsletter_empty_message(self):
+        url = reverse("emailing_subscribe_newsletter")
+        
+        data = {
+            'entity_type': 0,
+            'lastname': 'Dupond',
+            'firstname': 'Pierre',
+            'email': "pierre.dupond@mon-mail.fr",
+            'message': "",
+        }
+        self._patch_with_captcha(url, data)
+        
+        self.assertEqual(models.Contact.objects.count(), 0)
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(200, response.status_code)
+        
+        self.assertEqual(models.Contact.objects.count(), 1)
+        contact = models.Contact.objects.all()[0]
+        
+        self.assertEqual(contact.email, data["email"])
+        self.assertEqual(contact.lastname, data["lastname"])
+        self.assertEqual(contact.firstname, data["firstname"])
+        
+        self.assertEqual(0, contact.action_set.count())
+        
+        self.assertEqual(len(mail.outbox), 2) #email verification
+        
+        verification_email = mail.outbox[1]
+        self.assertEqual(verification_email.to, [contact.email]) #email verification
+        url = reverse('emailing_email_verification', args=[contact.uuid])
+        email_content = verification_email.message().as_string().decode('utf-8')
+        self.assertTrue(email_content.find(url)>0) #email verification
+        
+        notification_email = mail.outbox[0]
+        self.assertEqual(notification_email.to, [settings.SANZA_NOTIFICATION_EMAIL])
+        
+        
+    def test_subscribe_newsletter_no_entity(self):
+        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
+        
+        url = reverse("emailing_subscribe_newsletter")
+        
+        data = {
+            'entity_type': 0,
+            'lastname': 'Dupond',
+            'firstname': 'Pierre',
+            'groups': str(group1.id),
             'email': 'pdupond@apidev.fr',
         }
         self._patch_with_captcha(url, data)
@@ -673,7 +560,7 @@ class SubscribeTest(TestCase):
         
         self.assertEqual(contact.lastname, data['lastname'])
         self.assertEqual(contact.firstname, data['firstname'])
-        self.assertEqual(list(contact.entity.group_set.all()), [self.group1])
+        self.assertEqual(list(contact.entity.group_set.all()), [group1])
         
         self.assertEqual(len(mail.outbox), 2) #email verification
         
@@ -688,6 +575,9 @@ class SubscribeTest(TestCase):
         
         
     def test_subscribe_newsletter_entity(self):
+        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
+        group2 = mommy.make(models.Group, name="DEF", subscribe_form=True)
+        
         url = reverse("emailing_subscribe_newsletter")
         
         entity_type = mommy.make(models.EntityType, name='Pro', subscribe_form=True)
@@ -698,7 +588,7 @@ class SubscribeTest(TestCase):
             'lastname': 'Dupond',
             'firstname': 'Pierre',
             'email': 'pdupond@apidev.fr',
-            'groups': [self.group1.id, self.group2.id],
+            'groups': [group1.id, group2.id],
         }
         self._patch_with_captcha(url, data)
         
@@ -715,7 +605,7 @@ class SubscribeTest(TestCase):
         self.assertEqual(contact.entity.name, data['entity'])
         self.assertEqual(contact.lastname, data['lastname'])
         self.assertEqual(contact.firstname, data['firstname'])
-        self.assertEqual(list(contact.entity.group_set.all()), [self.group1, self.group2])
+        self.assertEqual(list(contact.entity.group_set.all()), [group1, group2])
         
         self.assertEqual(len(mail.outbox), 2) #email verification
         
@@ -729,6 +619,9 @@ class SubscribeTest(TestCase):
         self.assertEqual(notification_email.to, [settings.SANZA_NOTIFICATION_EMAIL])
         
     def test_subscribe_newsletter_private_group(self):
+        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
+        group2 = mommy.make(models.Group, name="DEF", subscribe_form=False)
+        
         url = reverse("emailing_subscribe_newsletter")
         
         entity_type = mommy.make(models.EntityType, name='Pro', subscribe_form=True)
@@ -738,7 +631,7 @@ class SubscribeTest(TestCase):
             'lastname': 'Dupond',
             'firstname': 'Pierre',
             'email': 'pdupond@apidev.fr',
-            'groups': [self.group1.id, self.group3.id],
+            'groups': [group1.id, group2.id],
         }
         self._patch_with_captcha(url, data)
         
@@ -853,4 +746,137 @@ class SubscribeTest(TestCase):
         self.assertEqual(404, response.status_code)
         contact = models.Contact.objects.get(id=contact.id)
         self.assertEqual(contact.email_verified, False)
-    
\ No newline at end of file
+        
+        
+        
+class EmailTrackingTest(BaseTestCase):
+    
+    def setUp(self):
+        super(EmailTrackingTest, self).setUp()
+        settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'
+        settings.COOP_CMS_REPLY_TO = 'titi@toto.fr'
+        
+        settings.COOP_CMS_SITE_PREFIX = "toto.fr"
+        site = Site.objects.get_current()
+        site.domain = "toto.fr"
+        site.save()
+        
+    def test_track_image(self):
+            
+        entity = mommy.make(models.Entity, name="my corp")
+        
+        names = ['alpha', 'beta', 'gamma']
+        contacts = [mommy.make(models.Contact, entity=entity,
+            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
+        
+        newsletter_data = {
+            'subject': 'This is the subject',
+            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a><a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>',
+            'template': 'test/newsletter_contact.html'
+        }
+        newsletter = mommy.make(Newsletter, **newsletter_data)
+        
+        emailing = mommy.make(Emailing,
+            newsletter=newsletter, status=Emailing.STATUS_SENT,
+            scheduling_dt = datetime.now(), sending_dt = datetime.now())
+        for c in contacts:
+            emailing.sent_to.add(c)
+        emailing.save()
+        
+        self.assertEqual(emailing.opened_emails.count(), 0)
+        
+        for c in contacts[:-1]:
+            tracking_url = reverse("emailing_email_tracking", args=[emailing.id, c.uuid])
+            response = self.client.get(tracking_url)
+            self.assertEqual(response.status_code, 200)
+        
+        self.assertEqual(emailing.opened_emails.count(), len(contacts)-1)
+        for c in contacts[:-1]:
+            self.assertTrue(c in list(emailing.opened_emails.all()))
+        
+        for c in contacts[-1:]:
+            self.assertFalse(c in list(emailing.opened_emails.all()))
+            
+    def test_track_image_twice(self):
+            
+        entity = mommy.make(models.Entity, name="my corp")
+        
+        names = ['alpha', 'beta', 'gamma']
+        contacts = [mommy.make(models.Contact, entity=entity,
+            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
+        
+        newsletter_data = {
+            'subject': 'This is the subject',
+            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a><a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>',
+            'template': 'test/newsletter_contact.html'
+        }
+        newsletter = mommy.make(Newsletter, **newsletter_data)
+        
+        emailing = mommy.make(Emailing,
+            newsletter=newsletter, status=Emailing.STATUS_SENT,
+            scheduling_dt = datetime.now(), sending_dt = datetime.now())
+        for c in contacts:
+            emailing.sent_to.add(c)
+        emailing.save()
+        
+        self.assertEqual(emailing.opened_emails.count(), 0)
+        
+        for c in contacts[:-1]:
+            tracking_url = reverse("emailing_email_tracking", args=[emailing.id, c.uuid])
+            response = self.client.get(tracking_url)
+            self.assertEqual(response.status_code, 200)
+            
+            response = self.client.get(tracking_url)
+            self.assertEqual(response.status_code, 200)
+        
+        self.assertEqual(emailing.opened_emails.count(), len(contacts)-1)
+        for c in contacts[:-1]:
+            self.assertTrue(c in list(emailing.opened_emails.all()))
+        
+        for c in contacts[-1:]:
+            self.assertFalse(c in list(emailing.opened_emails.all()))
+            
+    def test_send_newsletter_check_tracking(self):
+            
+        entity = mommy.make(models.Entity, name="my corp")
+        
+        names = ['alpha', 'beta', 'gamma']
+        contacts = [mommy.make(models.Contact, entity=entity,
+            email=name+'@toto.fr', lastname=name.capitalize()) for name in names]
+        
+        newsletter_data = {
+            'subject': 'This is the subject',
+            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a><a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>',
+            'template': 'test/newsletter_contact.html'
+        }
+        newsletter = mommy.make(Newsletter, **newsletter_data)
+        
+        emailing = mommy.make(Emailing,
+            newsletter=newsletter, status=Emailing.STATUS_SCHEDULED,
+            scheduling_dt = datetime.now(), sending_dt = None)
+        for c in contacts:
+            emailing.send_to.add(c)
+        emailing.save()
+        
+        management.call_command('emailing_scheduler', verbosity=0, interactive=False)
+        
+        emailing = Emailing.objects.get(id=emailing.id)
+        
+        #check emailing status
+        self.assertEqual(emailing.status, Emailing.STATUS_SENT)
+        self.assertNotEqual(emailing.sending_dt, None)
+        self.assertEqual(emailing.send_to.count(), 0)
+        self.assertEqual(emailing.sent_to.count(), len(contacts))
+        
+        self.assertEqual(len(mail.outbox), len(contacts))
+        
+        outbox = list(mail.outbox)
+        outbox.sort(key=lambda e: e.to)
+        contacts.sort(key=lambda c: c.get_email)
+        
+        for email, contact in zip(outbox, contacts):
+            self.assertNotEqual(newsletter.get_site_prefix(), "")
+            tracking_url = newsletter.get_site_prefix() + reverse("emailing_email_tracking",
+                args=[emailing.id, contact.uuid])
+            self.assertTrue(email.alternatives[0][1], "text/html")
+            self.assertTrue(email.alternatives[0][0].find(tracking_url)>=0)
