@@ -1,5 +1,10 @@
 # -*- coding: utf-8 -*-
 
+from django.conf import settings
+if 'localeurl' in settings.INSTALLED_APPS:
+    from localeurl.models import patch_reverse
+    patch_reverse()
+    
 from django.test import TestCase
 from django.contrib.auth.models import User, Permission
 from django.core.urlresolvers import reverse
@@ -16,15 +21,22 @@ from django.template import Template, Context
 from utils import create_profile_contact, notify_registration
 from registration.models import RegistrationProfile
 import os.path
-
+from unittest import skipIf
+from django.contrib.contenttypes.models import ContentType
+import logging
+        
 class BaseTestCase(TestCase):
 
     def setUp(self):
+        logging.disable(logging.CRITICAL)
         self.user = User.objects.create(username="toto")
         self.user.set_password("abc")
         self.user.save()
         self._login()
         
+    def tearDown(self):
+        logging.disable(logging.NOTSET)
+        
     def _get_file(self, file_name='unittest1.txt'):
         full_name = os.path.normpath(os.path.dirname(__file__) + '/fixtures/' + file_name)
         return open(full_name, 'rb')
@@ -42,6 +54,13 @@ class IfProfilePermTemplateTagsTest(BaseTestCase):
                 self.user = user
         return DummyRequest(self.user)
     
+    def setUp(self):
+        super(IfProfilePermTemplateTagsTest, self).setUp()
+        ct = ContentType.objects.get_for_model(get_article_class())
+        perm = Permission.objects.get(codename='change_article', content_type=ct)
+        self.user.user_permissions.add(perm)
+        self.user.save()
+    
     def test_create_article(self):
         tpl = Template('{% load sanza_profile_perm %}{% if_can_do_article "test" %}HELLO{% endif %}')
         html = tpl.render(Context({'request': self._request()}))
@@ -321,8 +340,9 @@ class DownloadTestCase(BaseTestCase):
         self.client.logout()
         response = self.client.get(doc.get_download_url(), follow=True)
         self.assertEqual(response.status_code, 200)
+        
         redirect_url = response.redirect_chain[-1][0]
-        login_url = reverse('django.contrib.auth.views.login')
+        login_url = reverse('auth_login')
         self.assertTrue(redirect_url.find(login_url)>0)
         
     def test_download_private_not_in_group(self):
@@ -477,7 +497,7 @@ class ProfileBackendTest(TestCase):
         contact.entity.default_contact.delete()
         self._create_profile_and_check(user)
         self.assertEqual(models.Contact.objects.count(), 1)
-        self.assertEqual(models.Action.objects.count(), 0)
+        self.assertEqual(models.Action.objects.count(), 1)#account
         
         
     def test_create_sanza_contact_multiple_email(self):
@@ -493,9 +513,9 @@ class ProfileBackendTest(TestCase):
         self.assertEqual(contact.firstname, user.first_name)
         
         self.assertEqual(models.Contact.objects.count(), 3)
-        self.assertEqual(models.Action.objects.count(), 1)
-        action = models.Action.objects.all()[0]
-        self.assertEqual(action.contact, contact)
+        self.assertEqual(models.Action.objects.count(), 2) # wran duplicates + account creation
+        for action in models.Action.objects.all():
+            self.assertEqual(list(action.contacts.all()), [contact])
     
     def test_notifiy_registration(self):
         user = self._create_user()
@@ -511,15 +531,17 @@ class ProfileBackendTest(TestCase):
        
 class RegisterTestCase(TestCase):
 
-    def test_register_with_very_long_email(self):
+    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
+    def test_register(self):
         url = reverse('registration_register')
         data = {
-            'email': ('a'*30)+'@toto.fr',
+            'email': 'john@toto.fr',
             'password1': 'PAss',
             'password2': 'PAss',
             'accept_termofuse': True,
             'accept_newsletter': True,
             'accept_3rdpart': True,
+            'gender': 1,
         }
         response = self.client.post(url, data=data, follow=True)
         self.assertEqual(response.status_code, 200)
@@ -538,5 +560,103 @@ class RegisterTestCase(TestCase):
         self.assertEqual(response.status_code, 200)
         contact = models.Contact.objects.get(email=data['email'])
         self.assertEqual(contact.accept_newsletter, data['accept_newsletter'])
+        self.assertEqual(contact.gender, data['gender'])
         
         self.assertTrue(self.client.login(email=data['email'], password=data['password1']))
+        
+    @skipIf(not ("registration" in settings.INSTALLED_APPS), "registration not installed")
+    def test_register_no_gender(self):
+        url = reverse('registration_register')
+        data = {
+            'email': 'john@toto.fr',
+            'password1': 'PAss',
+            'password2': 'PAss',
+            'accept_termofuse': True,
+            'accept_newsletter': True,
+            'accept_3rdpart': True,
+            
+        }
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+        user = User.objects.get(email=data['email'])
+        
+        self.assertEqual(RegistrationProfile.objects.count(), 1)
+        rp = RegistrationProfile.objects.all()[0]
+        self.assertEqual(rp.user, user)
+        activation_url = reverse('registration_activate', args=[rp.activation_key])
+        
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].to, [data['email']])
+        self.assertTrue(mail.outbox[0].body.find(activation_url))
+        
+        response = self.client.get(activation_url, follow=True)
+        self.assertEqual(response.status_code, 200)
+        contact = models.Contact.objects.get(email=data['email'])
+        self.assertEqual(contact.accept_newsletter, data['accept_newsletter'])
+        self.assertEqual(contact.gender, 0)
+        
+        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))
+        
+    def test_register_with_very_long_email(self):
+        url = reverse('registration_register')
+        data = {
+            'email': ('a'*30)+'@toto.fr',
+            'password1': 'PAss',
+            'password2': 'PAss',
+            'accept_termofuse': True,
+            'accept_newsletter': True,
+            'accept_3rdpart': True,
+            'gender': 1,
+        }
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+        user = User.objects.get(email=data['email'])
+        
+        self.assertEqual(RegistrationProfile.objects.count(), 1)
+        rp = RegistrationProfile.objects.all()[0]
+        self.assertEqual(rp.user, user)
+        activation_url = reverse('registration_activate', args=[rp.activation_key])
+        
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].to, [data['email']])
+        self.assertTrue(mail.outbox[0].body.find(activation_url))
+        
+        response = self.client.get(activation_url, follow=True)
+        self.assertEqual(response.status_code, 200)
+        contact = models.Contact.objects.get(email=data['email'])
+        self.assertEqual(contact.accept_newsletter, data['accept_newsletter'])
+        self.assertEqual(contact.gender, data['gender'])
+        
+        self.assertTrue(self.client.login(email=data['email'], password=data['password1']))
+        
+    def test_register_no_password(self):
+        url = reverse('registration_register')
+        data = {
+            'email': 'toto@toto.fr',
+            'password1': '',
+            'password2': '',
+            'accept_termofuse': True,
+            'accept_newsletter': True,
+            'accept_3rdpart': True,
+            'gender': 1,
+        }
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
+        
+    def test_register_different_passwords(self):
+        url = reverse('registration_register')
+        data = {
+            'email': 'toto@toto.fr',
+            'password1': 'ABC',
+            'password2': 'DEF',
+            'accept_termofuse': True,
+            'accept_newsletter': True,
+            'accept_3rdpart': True,
+            'gender': 1,
+        }
+        response = self.client.post(url, data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
+        
+        
\ No newline at end of file
